
'Solar Analysis Tools -
 'Copyright (C) 2005 Gary Reysa (gary@BuildItSolar.com)

 ' This program is free software; you can redistribute it and/or modify it
 ' under the terms of the GNU General Public License as published by the Free
 ' Software Foundation; either version 2 of the License, or (at your option)
 ' any later version.

 ' This program is distributed in the hope that it will be useful and encourage
 ' further development of solar tools, but   WITHOUT ANY WARRANTY; without even 
 ' the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 ' See the GNU General Public License   for more details.


' CollectorEcurveCls
' This class models a solar collector in which the efficiency 
' curve for the collector is known.
' It inherits from CollectorCls
' Adds Inputs:
'   YIntercept -- the Y intercept on the collector efficiency curve (ie max efic)
'   Slope      -- the slope of the collector curve in (BTU/hr-ft^2-F) (input as a negative number)
'   Ti      -- collector fluid inlet temperature (F)
'   Ta      -- ambient temperature (F)
' Adds Outputs:
'   ColEfic  -- collector efficiency (ratio of collector output/solar input
'   Qout     -- collector output in BTU/hr (defined as ColEfic * SolarInput)
'
' 11/08/05 
' Appears to be working OK -- gives general agreement with SRCC data -- should do more detailed checking.

Imports System.Math
Imports System.IO


Public Class CollectorEcurveCls
Inherits CollectorCls

Private mTi As Double ' collector inlet temp (F)
Private mTa As Double ' ambient temp (F)
Private mYIntercept As Double  ' Y intercept on efic curve (no units)
Private mSlope As Double  ' Slope of collector efficiency curve (BTU/hr-ft^2-F)

Private mEfic As Double ' collector efficiency
Private mQout As Double    ' power output of collector (BTU/hr)

' New input properties:
' Collector inlet temperature (F)
Public Property Ti() As Double
Get
    Return mTi
End Get
Set(ByVal Value As Double)
    mTi = Value
    ReCalcECurve()
End Set
End Property

' Ambient temperature (F)
Public Property Ta() As Double
Get
    Return mTa
End Get
Set(ByVal Value As Double)
    mTa = Value
    ReCalcECurve()
End Set
End Property

' Slope of the collector efficiency curve (BTU/hr-ft^2-F) -- this will normally be a negative number
Public Property Slope() As Double
Get
    Return mSlope
End Get
Set(ByVal Value As Double)
    mSlope = Value
    ReCalcECurve()
End Set
End Property


' Y (efficiency) intercept of the collector efficiency curve (no units)
' This is the efficiency of the collector when the parameter (Ti-Ta)/Isun = 0
Public Property YIntercept() As Double
Get
    Return mYIntercept
End Get
Set(ByVal Value As Double)
    mYIntercept = Value
    ReCalcECurve()
End Set
End Property

'New Output Properties:
' Efficiency of the collector
ReadOnly Property Efic() As Double
Get
    Return mEfic
End Get
End Property

' Output of the collector in (BTU/hr)
ReadOnly Property QOut() As Double
Get
    Return mQout
End Get
End Property

Sub New()
    ' how do I run parent constructor? -- seems to get run automatically
    mSlope = -1.0
    mYIntercept = 0.8
    mTi = 120.0
    mTa = 30.0
End Sub

Private Sub ReCalcECurve()
    ' call parent ReCalc
    ReCalc()
    ' now calc new parms
    Dim parm As Double
    If Icoltot < 0.1 Then
        parm = 100
    Else
         parm = (mTi - mTa) / mIcoltot
    End If

    mEfic = mYIntercept + parm * mSlope
    If mEfic < 0 Then mEfic = 0
    mQout = mEfic * mIcoltot * mArea

End Sub

Public Sub DumpEC()
    Dump()
    Console.WriteLine("  Eficiency              {0,6:F3}", mEfic)
    Console.WriteLine("  Qout                   {0,8:F0}", mQout)
    Console.WriteLine(" Yinter, Slope           {0,6:F3} {1,7:F3}", mYIntercept, mSlope)

End Sub

End Class
